#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## lib_infodisks.pm
## 
## ######### PROJECT DESCRIPTION : ###
## 
## 
## 
## ###################################
## 
## Made by Sardinha Mickael
## Login   <mike@intellique.com>
## 
## Started on  Mon Feb 26 15:54:54 2007 Sardinha Mickael
## Last update Wed Feb 13 18:50:36 2008 Boutonnet Alexandre
## 
## ###################################
##

package lib_infodisks;

# Declaration des Modules utilises
use strict;
use Data::Dumper;

# Definition des binaires utiliser
my $dev		= "/dev/";

my $known_fs = {
		"primary" => undef, 
		"logical" => undef,
		"extended" => undef,
		"primaire" => undef,
		"logique" => undef,
		"extendu" => undef,
		"" => undef,
		};

# On set la variable avec le path de fdisk
my $cmd_fdisk		= _test_cmd("fdisk", "/sbin/fdisk");
my $cmd_parted		= _test_cmd("parted","/sbin/parted");

my $error_parted	= "Error: Wrong version of Parted. \n";


my $regex	= qr!^\s*(\d+)\s+(\d+)[Bk]\s+(\d+)B\s+(\d+)B\s+\w+\s+([\-0-9a-zA-Z_]*)\s*! ;
my $regex2	= qr!^\s*(\d+)\s+(\d+)B\s+(\d+)B\s+(\d+)B\s+([\-0-9a-zA-Z_]*)\s*! ;
my $regex3	= qr!^\s*(\d+)\s+(\d+)B\s+(\d+)B\s+(\d+)B\s+([\-0-9a-zA-Z_]*)\s*! ;

# Acquisition de certaines de donnees grace a fdisk qu'on place ensuite dans la
# Hash: type donnees - disk(ex hda) - head - sectors - cylinders
sub	get_disk_info {
    my @fdisk_all	= qx!$cmd_fdisk -l 2>/dev/null | grep Di!;
    my $flag1	= 0;
    my $flag2	= 0;
    my $hash;

    chomp ( my $version_parted = qx!$cmd_parted -v | cut -d " " -f 3 | cut -d "." -f 1-2! );

	my @list_disk ;
    foreach my $line (@fdisk_all) {
        #Disk /dev/sda: 249.9 GB, 249988907008 bytes
		if ( $line =~ /^\s*(?:Disk|Disque)\s+\/dev\/(.*):\s+.*\s+(\d*)\s+/) {
	    	push @list_disk, $dev.$1;
		}
	}
	
	foreach my $disk ( @list_disk ) {
		if ( $disk =~ /^\/dev\/(md.*)/ ) {
			my @info_parted = qx!$cmd_parted -s $disk unit B print!;
	    	_get_parted_info_md($hash, \@info_parted, $disk, $version_parted);

			next;
		}
		
		my @fdisk	= qx!$cmd_fdisk -l $disk 2>/dev/null!;

    	foreach my $line (@fdisk) {
        	#Disk /dev/sda: 249.9 GB, 249988907008 bytes
			if ( $line =~ /^\s*(?:Disk|Disque)\s+\/dev\/(.*):\s+.*\s+(\d*)\s+/) {
				# Si on n'obtient pas la taille par fdisk, on la recupere dans /proc/partitions
		   		if ( $2 == 0){
					my $temp = qx!grep $1\$ /proc/partitions !;
					my (undef,undef,$real_size) = split (' ', $temp); 
					$hash->{$disk}{'size'} = $real_size;
					$flag1 = 1 ;
	    		}
		    	else {
					$hash->{$disk}{'size'} = $2;
					$flag1 = 1 ;
		    	}
			}
			# 255 heads, 63 sectors/track, 30392 cylinders
			elsif ($line =~ /^\s*(\d*)\s+.*,\s*(\d*)\s+sect.*,\s*(\d*).*/) {
				$hash->{$disk}{'head'} = $1;
		    	$hash->{$disk}{'sectors'} = $2;
		    	$hash->{$disk}{'cylinders'} = $3;
	    		$flag2 = 1;
			}
			# Des que toutes les infos de fdisk sont recuperees, on recupere les infos de parted
			elsif ( $flag2 && $flag1)
			{
	    		my @info_parted = qx!$cmd_parted -s $disk unit B print 2>/dev/null!;
	    		_get_parted_info($hash, \@info_parted, $disk, $version_parted);
    			$flag1 = 0;
	    		$flag2 = 0;
			}
	
		}
    }
	
    return $hash;
}

# Fonction de recuperation de la liste des disques.
# __Attention__ cette methode passe outre les devices /dev/md*
sub get_disk_list
{
    my @return_tab = qx($cmd_fdisk -l 2> /dev/null | grep -e "Disque" -e "Disk" | grep -v md | cut -f1 -d ":" | cut -f2 -d " " | sort | uniq);
    foreach (@return_tab) {
	chomp ($_);
    }

    return (@return_tab);
}

# Fonction de recuperation des partitions en fonction d'un disque
# Le parametre $dev est le device sur lequel parted travaille
# S'il est omis, cette methode retour zero (0)
sub get_disk_partition
{
    my ($dev) = @_;

    return (0) if (!$dev);

    my @return_tab;
    my @info_parted = qx!$cmd_parted -s $dev unit B print 2>/dev/null!;

    # Ici je test si le retour de parted n'est pas une erreur
    # si c'est le cas je quitte en retournant zero
    return(0) if ( !@info_parted || $info_parted[0]  =~ m/^Error: /);

    my $flag = 0;
    my $regexp = 0;

    foreach my $line (@info_parted) {
	unless ($flag) {
	    if ($line =~ /^\s*(?:Number|Num.ro)/)
	    {
		($flag, $regexp) = _detect_parted_version($line);
		return (0) if (!$flag);
	    }
	}
	if ( $flag && ($line =~ /$regexp/) ) {
	    push(@return_tab, $dev.$1);
	}
    }
    
    return (@return_tab);
}

# Fonction de recuperation des infos propres a une partition
# Cette methode retourne une reference sur une hash
# Le parametre $part est le nom de la partition. S'il est absent, la fonction
# retourne zero (0)
sub get_partition_info
{
    my ($part) = @_;

    return (0) if (!$part);

    my $return_hash = {};
    my @info_parted = qx!$cmd_parted -s $part unit B print 2>/dev/null!;

    # Ici je test si le retour de parted n'est pas une erreur
    # si c'est le cas je quitte en retournant zero
    return (0) if (!@info_parted || $info_parted[0]  =~ m/^Error: /);

    my $flag = 0;
    my $regexp = 0;

    foreach my $line (@info_parted) {
	unless ($flag) {
	    if ($line =~ /^\s*(?:Number|Num.ro)/)
	    {
		($flag, $regexp) = _detect_parted_version($line);
		return (0) if (!$flag);
	    }
	}
	if ( $flag && ($line =~ /$regexp/) ) {
	    $return_hash->{$part}{'start'}	= $2 ;
	    $return_hash->{$part}{'end'}	= $3 ;
	    $return_hash->{$part}{'size'}	= $4 ;
	    
	    if ( exists $known_fs->{$5} ) {
			$return_hash->{$part}{'filesystem'}	= "Unspecified filesystem" ;
	    }
	    else {
			$return_hash->{$part}{'filesystem'}	= $5 ;
	    }
	}
    }
    return ($return_hash);
}

# Fonction de recuperation des infos detaillees sur un disque
# __ATTENTION__ Cette methode ne retourne pas d'info sur les partitions
# Cette methode prend en parametre un device et retourne zero dans deux cas :
# - le device n'existe pas
# - le parametre est absent
# Sinon elle retourne une reference sur une hash
sub get_one_disk_info
{
    my ($dev) = @_;

    return (0) if (!$dev);

    my $return_hash = {};
    my @info_parted = qx!$cmd_parted -s $dev unit B print 2>/dev/null!;

    # Ici je test si le retour de parted n'est pas une erreur
    # si c'est le cas je quitte en retournant zero
    return(0) if (!@info_parted || $info_parted[0]  =~ m/^Error: /);

    my @fdisk = qx!$cmd_fdisk -l $dev!;
    my $disk = 0;

    foreach my $line (@fdisk)
    {
        #Disk /dev/sda: 249.9 GB, 249988907008 bytes
	if ( $line =~ /^\s*(?:Disk|Disque)\s+\/dev\/(.*):\s+.*\s+(\d*)\s+/)
	{
	    if ( $2 == 0){
		my $temp = qx!grep $1\$ /proc/partitions !;
		my (undef,undef,$real_size) = split (' ', $temp); 
		$return_hash->{$dev}{'size'} = $real_size;
	    }
	    else {
		$return_hash->{$dev}{'size'} = $2;
	    }
	}
	# 255 heads, 63 sectors/track, 30392 cylinders
	elsif ($line =~ /^\s*(\d*)\s+.*,\s*(\d*)\s+sect.*,\s*(\d*).*/) {
	    $return_hash->{$dev}{'head'} = $1;
	    $return_hash->{$dev}{'sectors'} = $2;
	    $return_hash->{$dev}{'cylinders'} = $3;
	}
    }

    $return_hash->{$dev}{"model_name"} = "N.A.";

    # Maintenant, on passe a la recuperation du nom du disque... la c sport
    if ($dev =~ m/\/hd.$/)
    {
	# recuperation du DD que l'on gere
	my $dd = ($dev =~ m/\/dev\/(.{3})/)[0];

 	# recuperation de la liste des DD
 	my $champ = qx(ls /proc/ide/ide*/*/model | grep $dd 2> /dev/null);

 	# recuperation du path pour le /proc correspondant a notre DD
 	$champ =~ s/model\n//;

 	# recuperation du model_name..
	chop (my $tmp = _open_file_line($champ . "model"));
	$return_hash->{$dev}{"model_name"} = $tmp if ($tmp);
    }
    ####
    # Gestion sata, usb, scsi..
    ####
    elsif ($dev =~ m/\/sd.$/)
    {
	# recuperation de la liste des infos pour le $device que l'on gere
	my @tmp_model = qx(scsi_info $dev);

	# boucling sur le tableau dernierement recupere
	foreach (@tmp_model)
	{
	    # Si je match "MODEL="
	    if ($_ =~ m/MODEL=\".*/)
	    {
		# je recupere le model_name
		$return_hash->{$dev}{"model_name"} = [($_ =~ m/MODEL=\"(.*)\"/)[0], 0];
	    }
	}
    }

    # recuperation du label du disque.
    $return_hash->{$dev}{"label"} = qx($cmd_parted -s $dev p | grep -e "Partition Table:" -e "Table de partition :" 2>/dev/null);
    $return_hash->{$dev}{"label"} =~ s/.*: //;
    chomp($return_hash->{$dev}{"label"});

    return ($return_hash);
}

######################## Methodes privees ######################################

# Acquisition de certaines de donnees grace a parted qu'on place ensuite dans la
# Hash: type donnees - partition (ex hda1) - start - end - filesystem
sub	_get_parted_info
{
    my $regexp	= 0;
    my $flag	= 0;

    my ($hash, $parted, $disk, $version_parted) = @_;

    foreach my $line (@{$parted}) {
		unless ($flag) {
			# Detection de la version puis selection de la regexp correspondante
	    	if ($line =~ /^\s*(?:Number|Num.ro)/)	{
				($flag, $regexp) = _detect_parted_version($line);
				return (0) if (!$flag);
	    	}
		}
		#                 2       3800.000   4800.000  linux-swap
		if ( $flag && ($line =~ /$regexp/) ) {
		    $hash->{$disk}{'partition'}{$disk.$1}{'start'}	= $2 ;
		    $hash->{$disk}{'partition'}{$disk.$1}{'end'}	= $3 ;
	    	$hash->{$disk}{'partition'}{$disk.$1}{'size'}	= $4 ;
	    
		    if ( exists $known_fs->{$5} ) {
				$hash->{$disk}{'partition'}{$disk.$1}{'filesystem'}	= "Unspecified filesystem" ;
	    	}
		    else {
				$hash->{$disk}{'partition'}{$disk.$1}{'filesystem'}	= $5 ;
	    	}
		}
    }
}

# Acquisition de certaines de donnees grace a parted qu'on place ensuite dans la
# Hash: type donnees - partition (ex hda1) - start - end - filesystem
sub	_get_parted_info_md
{
    my $regexp	= 0;
    my $flag	= 0;

    my ($hash, $parted, $disk, $version_parted) = @_;

    foreach my $line (@{$parted}) {
		unless ($flag) {
			# Detection de la version puis selection de la regexp correspondante
	    	if ($line =~ /^\s*(?:Number|Num.ro)/)	{
				($flag, $regexp) = _detect_parted_version($line);
				return (0) if (!$flag);
	    	}
		}
		#                 2       3800.000   4800.000  linux-swap
		if ( $flag && ($line =~ /$regexp/) ) {
		    $hash->{'raid '.$disk}{'partition'}{$disk}{'start'}	= $2 ;
		    $hash->{'raid '.$disk}{'partition'}{$disk}{'end'}	= $3 ;
	    	$hash->{'raid '.$disk}{'partition'}{$disk}{'size'}	= $4 ;
#			$hash->{'raid '.$disk}{'size'}						= $4;
	    
		    if ( exists $known_fs->{$5} ) {
				$hash->{'raid '.$disk}{'partition'}{$disk}{'filesystem'}	= "Unspecified filesystem" ;
	    	}
		    else {
				$hash->{'raid '.$disk}{'partition'}{$disk}{'filesystem'}	= $5 ;
	    	}
		}
    }
}

# Test la presence et l'existence des commandes fdisk et parted
sub	_test_cmd
{
    my $cmd	= shift;
    my $name	= shift;
    my $func;
    
    chomp($func = `which $cmd`);
    
    if ( ! -e $func){
	$func = $name;
    }

    if ( ! -x $func){
	print "The command $cmd is not executable.\n";
	exit 0;
    }
    return $func;
}

# Cette methode renvoie un bool et la regexp pour le traitement
# du retour de parted si la version de parted est connue
# Elle prend en parametre une string..
# DRY.....
sub _detect_parted_version
{
    my ($line) = @_;

    my $flag = 0;
    my $regexp = 0;

    if ($line =~ /\s+Start\s+End\s+Size\s+File\s+system\s+Name\s+Flags/ ||
	$line =~ /\s+D.but\s+Fin\s+Taille\s+Syst.me\sde\sfichiers\s+Nom\s+Fanions/)
    {
	$regexp	= $regex2;
	$flag	= 1;
    }
    elsif ($line =~ /\s+Start\s+End\s+Size\s+Type\s+File\s+system\s+Flags/ )
    {
	$regexp	= $regex;
	$flag	= 1;
    }
    elsif ($line =~ /\s+Start\s+End\s+Size\s+File\s+system\s+Flags/ ||
	   $line =~ /\s+D.but\s+Fin\s+Taille\s+Syst.me\sde\sfichiers\s+Fanions/)
    {
	$regexp	= $regex3;
	$flag	= 1;
    }
    # Si on a pas une des versions attendue, on affiche un message d'erreur et on quitte
    else
    {
	print $error_parted;
    }
    return $flag, $regexp;
}

# Fonction de recuperation du contenu d'un fichier...
sub _open_file_line
{
    my $path = shift;
    
    if (open (FILES, $path))
    {
	my $file = "";
	while (<FILES>)
	{
	    $file .= $_;
	}
	close FILES;
	return $file;
    }
    print ("ERROR $path: $!\n");
    
    return undef;
}

1;
